package packageAPI;

import javacard.framework.*;
import javacard.security.*;

public class APIKey implements DESKey{
	
	// Attributs
	private boolean hasBeenInit; // Clef initialisee ?
	private short 	size;        // Taille de la clef
	private byte 	type;        // Type de la clef
	private byte[] 	key;         // Valeur de la clef
	private byte 	level;       // Level de la clef
	private short 	max_clock;   // Date de péremption de la clé

	// Constructeur de la classe 
	public APIKey() {
		// La clef n'est pas initialisee juste cree
		hasBeenInit = false;
		// On fixe la taille
		size = KeyBuilder.LENGTH_DES3_3KEY;
		// On fixe le type de la clef
		type = KeyBuilder.TYPE_DES;
		// On cree le tableau qui va contenir la valeur de la clef
		key = new byte[(short) (size / ((short) 8))];
		// On definit un level 0 (min) par defaut
		level = (byte) 0;
		// On definit une date de péremption 0 pour la clef
		max_clock = (short) 0;
	}
	
	// Methode pour intialiser une clef
	public void init3DES(byte[] arg0, short arg1, byte level, byte type, short tMax_clock) {
		// On fixe la valeur de la clef avec setKey
		this.setKey(arg0, arg1);
		// On ajuste le level
		this.level = level;
		// On ajuste le type
		this.type = type;
		// On enregistre la date de péremption
		this.max_clock = tMax_clock;
		// La clef est initialisee
		this.hasBeenInit = true;
	}
	
	// Pour copier dans key la valeur directement prise de arg0 a partir de arg1
	public void setKey(byte[] arg0, short arg1) throws CryptoException,NullPointerException, ArrayIndexOutOfBoundsException {
		// On teste les tailles...
		if (arg0.length >= (byte) (arg1 + this.getLength())) {
			// On copie dans key a partir de arg0[arg1]
			for (byte i = 0; i < this.getLength(); i++) {
				this.key[i] = arg0[(byte) (i + arg1)];
			}
		// On leve l'exception sinon
		} else {
			throw new ArrayIndexOutOfBoundsException();
		}	
	}
	
	// Pour reinitialiser une clef
	public void clearKey() {
		// On fixe tous les bits a 0
		for (byte i = 0; i < this.key.length; i++) {
			this.key[i] = 0;
		}
		// La clef n'est plus initialisee
		this.hasBeenInit = false;
	}
	
	// Accesseur taille
	public short getSize() {
		return this.size;
	}
	
	// Accesseur taille (en octet)
	public short getLength() {
		return (short) this.key.length;
	}

	// Accesseur type
	public byte getType() {
		return this.type;
	}

	// Accesseur boolean initialisation
	public boolean isInitialized() {
		return this.hasBeenInit;
	}
	
	// Accesseur level
	public byte getLevel() {
		return level;
	}
	
	// Accesseur date de péremption
	public short getMaxClock() {
		return max_clock;
	}
	
	// Accesseur Tableau valeur clef
	public byte[] getKey() {
		return key;
	}
	
	// Accesseur renvoyant le n-ieme octet de la clef
	public byte getKey(byte n) {
		return key[n];
	}

	// Accesseur copiant la clef dans arg0 a partir de l'offset arg1
	public byte getKey(byte[] arg0, short arg1) {
		// On recupere la taille du tableau qui contient la valeur de la clef
		short length = this.getLength();
		// On verifie les tailles : arg1 non negatif et arg1+length toujours dans le tableau.
		if ((arg1 >= 0) && (arg1 <= (short) (arg0.length - length))) {
			// Definition de l'indice
			short i;
			// On copie a partir de arg0[arg1]...
			for (i = 0; i < length; i++) {
				arg0[(byte) (i + arg1)] = key[i];
			}
			// On renvoie la valeur finale de l'indice
			return ((byte) i);
		// On leve l'exception si les tailles ne sont pas bonnes
		} else {
			throw new ISOException(ISO7816.SW_FILE_FULL);
		}
	}	
}
